#include <stdio.h>
#include <math.h>

/* target: 60 C
 load   fan
 0.651  1
*/

double air  = 25.0;        /* [C] air temp */
double temp = 25.0;        /* [C] starting temperature is room temperature */
double load = 0.2;         /* load percent generatinmg the heat */
double fan_rpm = 0.0;      /* current rpm of the fan in percent */
double fan_rpm_min = 0.2;  /* minimum fan rpm, if non-zero; anything between zero and this value will become this value */

double cm_sink = 40.0;
double cm_air  = 5.0;

void sim(void)
{
	double rpm, Q_out, Q_in;
	/* the device is heating heat sink with Q_in energy */
	Q_in = load * 1000.0;
	temp += Q_in / (cm_sink);

	/* calculate Q_out depending on fan rpm using a nonlinear fan function */
	rpm = fan_rpm;
	if ((rpm < fan_rpm_min) && (rpm != 0))
		rpm = fan_rpm_min;
	else if (rpm > 1.0)
		rpm = 1.0;
	Q_out = (log(rpm*3.0+1.0)+0.2);
//printf("rpm=%f Qin=%f Qout=%f\n", rpm, Q_in, Q_out);
	temp -= Q_out / (cm_air) * (temp - air);
	if (temp < air)
		temp = air;
}

int main(int argc, char *argv[])
{
	float f;
	unsigned long int last_t, t;
	char line[1024];


	last_t = time(NULL) - 2; /* make sure we have some iterations to run after the first control arrives */
	while(!(feof(stdin))) {
		/* read line by line */
		*line = 0;
		fgets(line, sizeof(line), stdin);

		/* float means control, anything else is just tick for the timer */
		if (sscanf(line, "%f", &f) == 1)
			fan_rpm = f;


		/* run as many iterations as many seconds passed after the last invocation */
		for(t = time(NULL); last_t < t; last_t++) {
			load += (double)(rand() % 2000 - 1000) / 10000.0;
			if (load < 0.0)
				load = 0.0;
			if (load > 1.0)
				load = 1.0;

			sim();
			printf("%f  %f %f\n", temp, load, fan_rpm);
			fflush(stdout);
		}
	}
}
